from contextlib import suppress

from twisted.conch.telnet import TelnetProtocol, TelnetTransport
from twisted.internet import reactor
from twisted.internet.protocol import Factory

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QTelnetServer(BaseServer):
    NAME = "telnet_server"
    DEFAULT_PORT = 23

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.random_servers = [
            "Ubuntu 18.04 LTS",
            "Ubuntu 16.04.3 LTS",
            "Welcome to Microsoft Telnet Server.",
        ]

    def server_main(self):
        _q_s = self

        class CustomTelnetProtocol(TelnetProtocol):
            _state = None
            _user = None
            _pass = None

            def connectionMade(self):  # noqa: N802
                self._state = None
                self._user = None
                self._pass = None
                self.transport.write(b"PC login: ")
                self._state = b"Username"
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data):  # noqa: N802
                data = data.strip()
                if self._state == b"Username":
                    self._user = data
                    self._state = b"Password"
                    self.transport.write(b"Password: ")
                elif self._state == b"Password":
                    username = check_bytes(self._user)
                    password = check_bytes(data)
                    peer = self.transport.getPeer()
                    _q_s.check_login(username, password, ip=peer.host, port=peer.port)
                    self.transport.loseConnection()
                else:
                    self.transport.loseConnection()

            def connectionLost(self, reason=None):  # noqa: N802,ARG002
                self._state = None
                self._user = None
                self._pass = None

        factory = Factory()
        factory.protocol = lambda: TelnetTransport(CustomTelnetProtocol)
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from telnetlib import Telnet as TTelnet

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            _username = _username.encode("utf-8")
            _password = _password.encode("utf-8")
            t = TTelnet(_ip, _port)
            t.read_until(b"login: ")
            t.write(_username + b"\n")
            t.read_until(b"Password: ")
            t.write(_password + b"\n")


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qtelnetserver = QTelnetServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qtelnetserver.run_server()



from contextlib import suppress

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QPJLServer(BaseServer):
    NAME = "pjl_server"
    DEFAULT_PORT = 9100

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.printer = b"Brother HL-L2360"
        self.template = {
            "ProductName": "Brother HL-L2360",
            "FormatterNumber": "Q910CHL",
            "PrinterNumber": "L2360",
            "ProductSerialNumber": "VNB1897514",
            "ServiceID": "20157",
            "FirmwareDateCode": "20051103",
            "MaxPrintResolution": "900",
            "ControllerNumber": "Q910CHL",
            "DeviceDescription": "Brother HL-L2360",
            "DeviceLang": "ZJS PJL",
            "TotalMemory": "6890816",
            "AvailableMemory": "3706526",
            "Personality": "0",
            "EngFWVer": "10",
            "IPAddress": "172.17.0.2",
            "HWAddress": "0025B395EA01",
        }

    def server_main(self):
        _q_s = self

        class Custompjlrotocol(Protocol):
            _state = None

            def connectionMade(self):  # noqa: N802
                self._state = 1
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data):  # noqa: N802
                # Control to PJL (Removed)
                data = data.replace(b"\x1b%-12345X", b"")
                if data.lower().startswith(b"@pjl echo"):
                    self.transport.write(b"@PJL " + data[10:] + b"\x1b")
                elif data.lower().startswith(b"@pjl info id"):
                    self.transport.write(b"@PJL INFO ID\r\n" + _q_s.printer + b"\r\n\x1b")
                elif data.lower().startswith(b"@pjl prodinfo"):
                    prodinfo = "\r\n".join([k + " = " + v for k, v in _q_s.template.items()])
                    self.transport.write(prodinfo.encode("utf-8") + b"\x1b")
                _q_s.log(
                    {
                        "action": "query",
                        "status": "success",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                        "data": {"command": check_bytes(data)},
                    }
                )
                self.transport.loseConnection()

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = None

        factory = Factory()
        factory.protocol = Custompjlrotocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):  # noqa: ARG002
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_STREAM

            _ip = ip or self.ip
            _port = port or self.port
            c = socket(AF_INET, SOCK_STREAM)
            c.sendto(b"\x1b%-12345X@PJL prodinfo", (_ip, _port))
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qpjlserver = QPJLServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        qpjlserver.run_server()



from __future__ import annotations

import pytest

from honeypots import QMemcacheServer
from .utils import (
    assert_connect_is_logged,
    connect_to,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "61211"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QMemcacheServer, "port": PORT}],
    indirect=True,
)
def test_memcache_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT) as connection:
        connection.send(b"stats\r\n")
        data, _ = connection.recvfrom(10000)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, stats = logs
    assert_connect_is_logged(connect, PORT)

    assert stats["action"] == "stats"

    assert b"STAT libevent 2.1.8-stable" in data



from __future__ import annotations

import pytest
import requests

from honeypots import QIPPServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    wait_for_server,
)

PORT = "50631"
SERVER_CONFIG = {
    "honeypots": {
        "ipp": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QIPPServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_ipp_server(server_logs):
    with wait_for_server(PORT):
        body = (
            b"\x02\x00\x00\x0b\x00\x01/p\x01G\x00\x12attributes-charset\x00\x05utf-8H\x00\x1b"
            b"attributes-natural-language\x00\x02enE\x00\x0bprinter-uri\x00\x15"
            b"ipp://127.0.0.1:631/D\x00\x14requested-attributes\x00\x03allD\x00\x00\x00\x12media-col-database\x03"
        )
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Content-Length": f"{len(body)}",
            "Host": f"{IP}:{PORT}",
            "Connection": "close",
        }
        requests.post(f"http://{IP}:{PORT}/", data=body, headers=headers)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, query = logs
    assert_connect_is_logged(connect, PORT)

    assert query["action"] == "query"
    assert query["data"] == {
        "request": (
            "VERSION 2.0|REQUEST 0x12f70|OPERATION Get-Printer-Attributes|GROUP "
            "operation-attributes-tag|ATTR attributes-charset utf-8|ATTR "
            "attributes-natural-language en|ATTR printer-uri ipp://127.0.0.1:631/D"
        )
    }



from __future__ import annotations

from contextlib import suppress

import pytest
from ldap3 import ALL, Connection, Server
from ldap3.core.exceptions import LDAPInsufficientAccessRightsResult

from honeypots import QLDAPServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50389"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QLDAPServer, "port": PORT}],
    indirect=True,
)
def test_ldap_server(server_logs):
    with wait_for_server(PORT), suppress(LDAPInsufficientAccessRightsResult):
        connection = Connection(
            Server(IP, port=int(PORT), get_info=ALL),
            authentication="SIMPLE",
            user=USERNAME,
            password=PASSWORD,
            check_names=True,
            lazy=False,
            client_strategy="SYNC",
            raise_exceptions=True,
        )
        connection.open()
        connection.bind()

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)



from contextlib import suppress

from twisted.internet import reactor
from twisted.internet.protocol import Factory, Protocol

from honeypots.base_server import BaseServer
from honeypots.helper import check_bytes, server_arguments

GSSResponse = 112  # password response message


class QPostgresServer(BaseServer):
    NAME = "postgres_server"
    DEFAULT_PORT = 5432

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomPostgresProtocol(Protocol):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self._variables = {}
                self._state = None

            def read_data_custom(self, data: bytes):
                _data = data.decode("utf-8")
                encoded_list = _data[8:-1].split("\x00")
                self._variables = dict(zip(*([iter(encoded_list)] * 2)))

            def read_password_custom(self, data: bytes):
                data = data.decode("utf-8")
                self._variables["password"] = data[5:].split("\x00")[0]

            def connectionMade(self):  # noqa: N802
                self._state = 1
                self._variables = {}
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def dataReceived(self, data: bytes):  # noqa: N802
                print(data.hex())
                if self._state == 1:
                    self._state = 2
                    self.transport.write(b"N")
                elif self._state == 2:  # noqa: PLR2004
                    self.read_data_custom(data)
                    self._state = 3
                    self.transport.write(b"R\x00\x00\x00\x08\x00\x00\x00\x03")
                elif self._state == 3:  # noqa: PLR2004
                    message_type = data[0]
                    print(data.hex())
                    if message_type == GSSResponse and "user" in self._variables:
                        self.read_password_custom(data)
                        username = check_bytes(self._variables["user"])
                        password = check_bytes(self._variables["password"])
                        peer = self.transport.getPeer()
                        _q_s.check_login(username, password, ip=peer.host, port=peer.port)
                    self.transport.loseConnection()
                else:
                    self.transport.loseConnection()

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = 1
                self._variables = {}

        factory = Factory()
        factory.protocol = CustomPostgresProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from socket import socket

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password

            s = socket()
            s.connect((_ip,_port))
            s.send(b'\x00\x00\x00\x08\x04\xd2\x16\x2f')
            s.recv(1024)
            s.send(b'\x00\x00\x00\x21\x00\x03\x00\x00\x75\x73\x65\x72\x00' + _username + b'\x00\x64\x61\x74\x61\x62\x61\x73\x65\x00' + _username + b'\x00\x00')
            s.recv(1024)
            s.send(b'\x70\x00\x00\x00\x09' + _password + b'\x00')
            s.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qpostgresserver = QPostgresServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qpostgresserver.run_server()



from __future__ import annotations

import pytest
from paramiko import AutoAddPolicy, SSHClient

from honeypots import QSSHServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = 50022
SERVER_CONFIG = {
    "honeypots": {
        "ssh": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QSSHServer, "port": str(PORT), "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_ssh_server(server_logs):
    with wait_for_server(PORT):
        ssh = SSHClient()
        ssh.set_missing_host_key_policy(AutoAddPolicy())
        ssh.connect(IP, port=PORT, username=USERNAME, password=PASSWORD)
        ssh.exec_command("ls")
        ssh.close()

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 3
    connect, login, command = logs
    assert_connect_is_logged(connect, str(PORT))

    assert command["action"] == "command"
    assert command["data"] == {"command": "ls"}
    assert login["action"] == "login"
    assert login["username"] == USERNAME



from __future__ import annotations

import pytest
import requests

from honeypots import QHTTPSServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
)

PORT = "50443"
SERVER_CONFIG = {
    "honeypots": {
        "https": {
            "options": ["capture_commands"],
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QHTTPSServer, "port": PORT, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_https_server(server_logs):
    with wait_for_server(PORT):
        url = f"https://{IP}:{PORT}"
        data = {"username": USERNAME, "password": PASSWORD}
        requests.post(f"{url}/login.html", verify=False, data=data)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 3
    connect, post, login = logs
    assert_connect_is_logged(connect, PORT)
    assert_login_is_logged(login)

    assert "data" in connect
    assert connect["data"]["uri"] == "/login.html"
    assert connect["data"]["method"] == "POST"

    assert post["action"] == "POST"



from __future__ import annotations

from contextlib import suppress

import pytest
from socket import socket

from honeypots import QPostgresServer
from .utils import (
    assert_connect_is_logged,
    assert_login_is_logged,
    IP,
    load_logs_from_file,
    PASSWORD,
    USERNAME,
    wait_for_server,
    connect_to
)

PORT = "55432"

@pytest.mark.parametrize(
    "server_logs",
    [{"server": QPostgresServer, "port": PORT}],
    indirect=True,
)
def test_postgres_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT) as connection:
        connection.send(b'\x00\x00\x00\x08\x04\xd2\x16\x2f')
        data, _ = connection.recvfrom(10000)
        connection.send(b'\x00\x00\x00\x21\x00\x03\x00\x00\x75\x73\x65\x72\x00' + USERNAME.encode() + b'\x00\x64\x61\x74\x61\x62\x61\x73\x65\x00' + USERNAME.encode() + b'\x00\x00')
        data, _ = connection.recvfrom(10000)
        connection.send(b'\x70\x00\x00\x00\x09' + PASSWORD.encode() + b'\x00')

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect_, login = logs
    assert_connect_is_logged(connect_, PORT)
    assert_login_is_logged(login)


from contextlib import suppress
from socket import socket, SHUT_RDWR, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from ssl import SSLContext, PROTOCOL_TLSv1_2, CERT_NONE
from struct import unpack
from threading import Thread

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    create_certificate,
    check_bytes,
)


class QRDPServer(BaseServer):
    NAME = "rdp_server"
    DEFAULT_PORT = 3389

    def server_main(self):  # noqa: C901
        _q_s = self

        class ConnectionHandle(Thread):
            def __init__(self, sock, key, cert):
                super().__init__()
                self.sock = sock
                self.key = key
                self.cert = cert

            def get_value(self, length, data):
                with suppress(Exception):
                    var = b""
                    for idx, byte in enumerate(data):
                        if byte == 0 and data[idx + 1] == 0:
                            break
                        if byte == 0:
                            continue
                        var += bytes([byte])
                    if length / 2 == len(var):
                        return var
                return b""

            def extract_cookie(self, data: bytes) -> bytes:
                return data[: data.find(b"\r\n")]

            def extract_creds(self, data: bytes):
                with suppress(Exception):
                    (
                        domain_length,
                        user_length,
                        password_length,
                        shell_length,
                        working_dir_length,
                    ) = unpack("HHHHH", data[27:37])
                    location = 37
                    _ = self.get_value(domain_length, data[location:])
                    location += domain_length + 2
                    user = self.get_value(user_length, data[location:])
                    location += user_length + 2
                    password = self.get_value(password_length, data[location:])
                    location += password_length + 2
                    _ = self.get_value(shell_length, data[location:])
                    location += shell_length + 2
                    _ = self.get_value(working_dir_length, data[location:])
                return user, password

            def run(self):
                # There is no good documentation on how RDP protocol works (It took a bit of time
                # to figure it out - Use b1105eb1-d1f7-414b-ad68-fd0c5a7823e4 test case)
                initiator = b"\x00\x06"
                with suppress(Exception):
                    _q_s.log(
                        {
                            "action": "connection",
                            "src_ip": self.sock.getpeername()[0],
                            "src_port": self.sock.getpeername()[1],
                        }
                    )
                    # Client X.224 Connection Request PDU

                    data = self.sock.recv(1024)
                    if b"Cookie" in data:
                        cookie = self.extract_cookie(data[11:]).decode(errors="replace")
                        _q_s.log(
                            {
                                "action": "stshash",
                                "mstshash": "success",
                                "src_ip": self.sock.getpeername()[0],
                                "src_port": self.sock.getpeername()[1],
                                "data": {"stshash": cookie},
                            }
                        )

                    # Server X.224 Connection Confirm PDU
                    # data[0] version
                    # 19 (x13) total len
                    # 14 (x0e) X.224 len
                    # TLS only \x02\x00\x08\x00\x01\x00\x00\x00

                    self.sock.send(
                        b"\x03\x00\x00\x13\x0e\xd0\x00\x00\x12\x34\x00\x02\x00\x08\x00\x01\x00\x00\x00"
                    )
                    ctx = SSLContext(PROTOCOL_TLSv1_2)
                    ctx.set_ciphers("RSA:!aNULL")
                    ctx.check_hostname = False
                    ctx.verify_mode = CERT_NONE
                    ctx.load_cert_chain(certfile=self.cert, keyfile=self.key)
                    self.sock = ctx.wrap_socket(
                        self.sock, server_side=True, do_handshake_on_connect=True
                    )

                    data = self.sock.recv(1024)

                    # MCS Connect Response PDU with GCC Conference Create Response
                    # \x03\x00\x00
                    # \x7c
                    # \x02\xf0\x80\x7f\x66\x74\x0a\x01\x00\x02\x01\x00\x30\x1a\x02\x01\x22\x02
                    # \x01\x03\x02\x01\x00\x02\x01\x01\x02\x01\x00\x02\x01\x01\x02\x03\x00\xff
                    # \xf8\x02\x01\x02\x04
                    # \x4e
                    # \x00\x05\x00\x14\x7c\x00\x01\x2a\x14\x76\x0a\x01\x01\x00\x01\xc0\x00\x4d
                    # \x63\x44\x6e
                    # \x38
                    # \x01\x0c SC_CORE
                    # \x0e\x00\x04\x00\x08\x00\x03\x00\x00\x00\x03\x00
                    # \x02\x0c SC_SECURITY
                    # \x0c\x00\x00\x00\x00\x00\x00\x00\x00\x00
                    # \x03\x0c SC_NET
                    # 03eb I/O channel
                    # 03ec rdpdr channel
                    # 03ed cliprdr channel
                    # 03ef rdpsnd channel
                    # \x10\x00\xeb\x03\x04\x00\xec\x03\xed\x03\xee\x03\xef\x03
                    # \x04\x0c SC_MCS_MSGCHANNEL
                    # \x06\x00\xf0\x03
                    # \x08\x0c SC_MULTITRANSPORT
                    # \x08\x00\x00\x00\x00\x00

                    self.sock.send(
                        b"\x03\x00\x00\x7c\x02\xf0\x80\x7f\x66\x74\x0a\x01\x00\x02\x01\x00\x30"
                        b"\x1a\x02\x01\x22\x02\x01\x03\x02\x01\x00\x02\x01\x01\x02\x01\x00\x02"
                        b"\x01\x01\x02\x03\x00\xff\xf8\x02\x01\x02\x04\x4e\x00\x05\x00\x14\x7c"
                        b"\x00\x01\x2a\x14\x76\x0a\x01\x01\x00\x01\xc0\x00\x4d\x63\x44\x6e\x38"
                        b"\x01\x0c\x0e\x00\x04\x00\x08\x00\x03\x00\x00\x00\x03\x00\x02\x0c\x0c"
                        b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x03\x0c\x10\x00\xeb\x03\x04\x00"
                        b"\xec\x03\xed\x03\xee\x03\xef\x03\x04\x0c\x06\x00\xf0\x03\x08\x0c\x08"
                        b"\x00\x00\x00\x00\x00"
                    )

                    data = self.sock.recv(1024)
                    data = self.sock.recv(1024)

                    # Server MCS Attach-User Confirm PDU
                    # 03 00 00 0b 02 f0 80 2e 00 00 06
                    self.sock.send(b"\x03\x00\x00\x0b\x02\xf0\x80\x2e\x00" + initiator)

                    # Multiple channel join
                    # 03 00 00 0c 02 f0 80 38 00 06 03 eb
                    # 03 00 00 0f 02 f0 80 3e 00 00 06 03 eb 03 eb

                    with suppress(Exception):
                        # 7 times + 1
                        for _ in range(8):
                            data = self.sock.recv(1024)
                            if len(data) > 14:  # noqa: PLR2004
                                if data[15] == 64:  # noqa: PLR2004
                                    username, password = self.extract_creds(data)
                                    peer = self.sock.getpeername()
                                    _q_s.check_login(
                                        check_bytes(username),
                                        check_bytes(password),
                                        ip=peer.host,
                                        port=peer.port,
                                    )
                                    break
                            else:
                                self.sock.send(
                                    b"\x03\x00\x00\x0f\x02\xf0\x80\x3e\x00"
                                    + initiator
                                    + b"\x03"
                                    + bytes([data[-1]])
                                    + b"\x03"
                                    + bytes([data[-1]])
                                )

                    # MCS Disconnect Provider Ultimatum PDU
                    self.sock.send(b"\x03\x00\x00\x09\x02\xf0\x80\x21\x80")

                with suppress(Exception):
                    self.sock.shutdown(SHUT_RDWR)
                with suppress(Exception):
                    self.sock.close()

        rpdserver = socket(AF_INET, SOCK_STREAM)
        rpdserver.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        rpdserver.bind((self.ip, self.port))
        rpdserver.listen()

        with create_certificate() as (cert, key):
            while True:
                with suppress(Exception):
                    client, addr = rpdserver.accept()
                    client.settimeout(10.0)
                    ConnectionHandle(client, key, cert).start()

    def test_server(self, ip=None, port=None):
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_STREAM

            _ip = ip or self.ip
            _port = port or self.port
            c = socket(AF_INET, SOCK_STREAM)
            c.sendto(b"test", (_ip, _port))
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qrdpserver = QRDPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        qrdpserver.run_server()



from __future__ import annotations

import pytest

from honeypots import QSIPServer
from .utils import connect_to, IP, load_logs_from_file, wait_for_server

PORT = "55060"
EXPECTED_KEYS = ("action", "server", "src_ip", "src_port", "timestamp")
CALL_ID = "1@0.0.0.0"
CONTACT = "sip:user_3@test.test.test"
FROM = f"{CONTACT};tag=none"
TO = "<sip:user_2@test.test>"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QSIPServer, "port": PORT}],
    indirect=True,
)
def test_sip_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT, udp=True) as connection:
        payload = (
            "INVITE sip:user_1@test.test SIP/2.0\r\n"
            f"To: {TO}\r\n"
            f"From: {FROM}\r\n"
            f"Call-ID: {CALL_ID}\r\n"
            "CSeq: 1 INVITE\r\n"
            f"Contact: {CONTACT}\r\n"
            "Via: SIP/2.0/TCP 0.0.0.0;branch=34uiddhjczqw3mq23\r\n"
            "Content-Length: 1\r\n"
            "\r\n"
            "T"
        )
        connection.send(payload.encode())

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, request = logs

    assert all(k in connect for k in EXPECTED_KEYS)
    assert connect["action"] == "connection"
    assert connect["server"] == "sip_server"

    assert request["action"] == "request"
    assert request["src_ip"] == IP
    assert "data" in request
    assert request["data"]["call-id"] == CALL_ID
    assert request["data"]["contact"] == CONTACT
    assert request["data"]["from"] == FROM
    assert request["data"]["to"] == TO



from scapy.error import Scapy_Exception
from scapy.layers.snmp import SNMP
from twisted.internet import reactor
from twisted.internet.protocol import DatagramProtocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
)


class QSNMPServer(BaseServer):
    NAME = "snmp_server"
    DEFAULT_PORT = 161

    def server_main(self):
        _q_s = self

        class CustomDatagramProtocolProtocol(DatagramProtocol):
            def parse_snmp(self, data):
                try:
                    parsed_snmp = SNMP(data)
                    community = parsed_snmp.community.val
                    version = parsed_snmp.version.val
                    oids = " ".join([item.oid.val for item in parsed_snmp.PDU.varbindlist])
                except Scapy_Exception:
                    version = "UnKnown"
                    community = "UnKnown"
                    oids = "UnKnown"
                return version, community, oids

            def datagramReceived(self, data, addr):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "status": "fail",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                    }
                )
                version, community, oids = self.parse_snmp(data)
                if version or community or oids:
                    _q_s.log(
                        {
                            "action": "query",
                            "status": "success",
                            "src_ip": addr[0],
                            "src_port": addr[1],
                            "data": {"version": version, "community": community, "oids": oids},
                        }
                    )
                    self.transport.write(b"Error", addr)

        reactor.listenUDP(
            port=self.port, protocol=CustomDatagramProtocolProtocol(), interface=self.ip
        )
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):  # noqa: ARG002
        try:
            from pysnmp.hlapi import (
                getCmd,
                SnmpEngine,
                CommunityData,
                UdpTransportTarget,
                ContextData,
                ObjectType,
                ObjectIdentity,
            )

            _ip = ip or self.ip
            _port = port or self.port
            g = getCmd(
                SnmpEngine(),
                CommunityData("public"),
                UdpTransportTarget((_ip, _port)),
                ContextData(),
                ObjectType(ObjectIdentity("1.3.6.1.4.1.9.9.618.1.4.1.0")),
            )
            next(g)
        except Exception as error:
            self.logger.exception(f"Error during test of {self.NAME} server: {error}")


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QSNMPServer = QSNMPServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        QSNMPServer.run_server()



import struct
from contextlib import suppress
from struct import pack
from time import time

from twisted.internet import reactor
from twisted.internet.protocol import DatagramProtocol

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
)


class QNTPServer(BaseServer):
    NAME = "ntp_server"
    DEFAULT_PORT = 123

    def server_main(self):
        _q_s = self

        class CustomDatagramProtocolProtocol(DatagramProtocol):
            def system_time_to_ntp(self, time_):
                i = int(time_ + 2208988800.0) << 32
                f = int(((time_ + 2208988800.0) - int(time_ + 2208988800.0)) * 4294967296)
                return i, f

            def ntp_to_system_time(self, time_):
                i = float(time_ >> 32) - 2208988800.0
                f = float(int(i) & 0xFFFFFFFF) / 4294967296
                return i, f

            def datagramReceived(self, data, addr):  # noqa: N802
                version = "UnKnown"
                mode = "UnKnown"
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                    }
                )
                try:
                    version = data[0] >> 3 & 0x7
                    mode = data[0] & 0x7
                    i, f = self.system_time_to_ntp(time())
                    response = pack(
                        "!B B B b I I I Q Q Q Q",
                        0 << 6 | 3 << 3 | 2,
                        data[1],
                        data[2],
                        data[3],
                        0,
                        0,
                        0,
                        0,
                        data[10],
                        0,
                        i + f,
                    )
                    self.transport.write(response, addr)
                    status = "success"
                except (struct.error, TypeError, IndexError):
                    status = "failed"

                _q_s.log(
                    {
                        "action": "query",
                        "status": status,
                        "src_ip": addr[0],
                        "src_port": addr[1],
                        "data": {"version": version, "mode": mode},
                    }
                )

        reactor.listenUDP(
            port=self.port, protocol=CustomDatagramProtocolProtocol(), interface=self.ip
        )
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):  # noqa: ARG002
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_DGRAM

            _ip = ip or self.ip
            _port = port or self.port
            c = socket(AF_INET, SOCK_DGRAM)
            c.sendto(b"\x1b" + 47 * b"\0", (_ip, _port))
            c.recvfrom(256)
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QNTPServer = QNTPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QNTPServer.run_server()



#!/usr/bin/env python3

from __future__ import annotations

import logging
import sys
from argparse import ArgumentParser, SUPPRESS, Namespace
from atexit import register
from functools import wraps
from json import JSONDecodeError, loads
from os import geteuid
from pathlib import Path
from signal import alarm, SIGALRM, SIGINT, signal, SIGTERM, SIGTSTP
from subprocess import Popen
from time import sleep
from typing import Any
from uuid import uuid4

from netifaces import ifaddresses, AF_INET, AF_LINK, interfaces
from psutil import net_io_counters, Process

from honeypots import (
    QSniffer,
    QDHCPServer,
    QDNSServer,
    QElasticServer,
    QFTPServer,
    QHTTPProxyServer,
    QHTTPSServer,
    QHTTPServer,
    QIMAPServer,
    QIPPServer,
    QIRCServer,
    QLDAPServer,
    QMSSQLServer,
    QMemcacheServer,
    QMysqlServer,
    QNTPServer,
    QOracleServer,
    QPJLServer,
    QPOP3Server,
    QPostgresServer,
    QRDPServer,
    QRedisServer,
    QSIPServer,
    QSMBServer,
    QSMTPServer,
    QSNMPServer,
    QSOCKS5Server,
    QSSHServer,
    QTelnetServer,
    QVNCServer,
    is_privileged,
    clean_all,
    setup_logger,
    set_up_error_logging,
)

all_servers = {
    "dhcp": QDHCPServer,
    "dns": QDNSServer,
    "elastic": QElasticServer,
    "ftp": QFTPServer,
    "httpproxy": QHTTPProxyServer,
    "https": QHTTPSServer,
    "http": QHTTPServer,
    "imap": QIMAPServer,
    "ipp": QIPPServer,
    "irc": QIRCServer,
    "ldap": QLDAPServer,
    "mssql": QMSSQLServer,
    "memcache": QMemcacheServer,
    "mysql": QMysqlServer,
    "ntp": QNTPServer,
    "oracle": QOracleServer,
    "pjl": QPJLServer,
    "pop3": QPOP3Server,
    "postgres": QPostgresServer,
    "rdp": QRDPServer,
    "redis": QRedisServer,
    "sip": QSIPServer,
    "smb": QSMBServer,
    "smtp": QSMTPServer,
    "snmp": QSNMPServer,
    "socks5": QSOCKS5Server,
    "ssh": QSSHServer,
    "telnet": QTelnetServer,
    "vnc": QVNCServer,
}

logger = set_up_error_logging()


class SignalFence:
    def __init__(self, signals_to_listen_on, interval=1):
        self.fence_up = True
        self.interval = interval

        for signal_to_listen_on in signals_to_listen_on:
            signal(signal_to_listen_on, self.handle_signal)

    def handle_signal(self, signum, frame):  # noqa: ARG002
        self.fence_up = False

    def wait_on_fence(self):
        while self.fence_up:
            sleep(self.interval)


class Termination:
    def __init__(self, strategy):
        self.strategy = strategy

    def await_termination(self):
        if self.strategy == "input":
            input("")
        elif self.strategy == "signal":
            SignalFence([SIGTERM, SIGINT, SIGTSTP]).wait_on_fence()
        else:
            raise Exception(f"Unknown termination strategy: {self.strategy}")


def timeout(seconds=10):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            def handle_timeout(signum, frame):  # noqa: ARG001
                raise TimeoutError()

            signal(SIGALRM, handle_timeout)
            alarm(seconds)
            try:
                result = func(*args, **kwargs)
            finally:
                alarm(0)
            return result

        return wrapper

    return decorator


@timeout(5)
def server_timeout(obj, name):
    try:
        logger.info(f"Start testing {name}")
        obj.test_server()
    except TimeoutError:
        logging.error(f"Timeout during test {name}")
    logger.info(f"Done testing {name}")


class HoneypotsManager:
    def __init__(self, options: Namespace, server_args: dict[str, str | int]):
        self.options = options
        self.server_args = server_args
        self.config_data = self._load_config() if self.options.config else {}
        self.auto = options.auto if geteuid() != 0 else False
        self.honeypots: list[tuple[Any, str, bool]] = []

    def main(self):
        logger.info("For updates, check https://github.com/qeeqbox/honeypots")
        if not is_privileged():
            logger.warning(
                "Using system or well-known ports requires higher privileges (E.g. sudo -E)"
            )

        if self.options.list:
            for service in all_servers:
                print(service)
        elif self.options.kill:
            clean_all()
        elif self.options.chameleon and self.config_data:
            self._start_chameleon_mode()
        elif self.options.setup:
            if self.options.sniffer:
                self._set_up_sniffer()
            self._set_up_honeypots()

    def _load_config(self):
        config_path = Path(self.options.config)
        if not config_path.is_file():
            logger.error(f'Config file "{config_path}" not found')
            sys.exit(1)
        try:
            config_data = loads(config_path.read_text())
            logger.info(f"Successfully loaded config file {config_path}")
            return config_data
        except FileNotFoundError:
            logger.error(f"Unable to load config file: File {config_path} not found")
            sys.exit(1)
        except JSONDecodeError as error:
            logger.error(f"Unable to parse config file as JSON: {error}")
            sys.exit(1)

    def _set_up_honeypots(self):  # noqa: C901
        register(_exit_handler)
        if self.options.termination_strategy == "input":
            logger.info("Use [Enter] to exit or python3 -m honeypots --kill")
        if self.options.config != "":
            logger.warning("config.json file overrides --ip, --port, --username and --password")
        if self.options.setup == "all":
            self._start_all_servers()
        else:
            self._start_configured_servers()

        running_honeypots = {"good": [], "bad": []}
        if len(self.honeypots) > 0:
            for _, server_name, status in self.honeypots:
                if not status:
                    running_honeypots["bad"].append(server_name)
                else:
                    running_honeypots["good"].append(server_name)

            if len(running_honeypots["good"]) > 0:
                running_servers = ", ".join(running_honeypots["good"])
                logger.info(f"servers {running_servers} running...")

            if len(running_honeypots["bad"]) > 0:
                not_running_servers = ", ".join(running_honeypots["bad"])
                logger.warning(f"servers {not_running_servers} not running...")

            if len(running_honeypots["bad"]) == 0:
                logger.info("Everything looks good!")

            if len(running_honeypots["good"]) > 0 and not self.options.test:
                Termination(self.options.termination_strategy).await_termination()

            self._stop_servers()

    def _start_all_servers(self):
        try:
            for service in all_servers:
                self._start_server(service)
        except Exception as error:
            logger.exception(f"Starting honeypots failed: {error}")

    def _start_configured_servers(self):
        for service in self.options.setup.split(","):
            logger.info("Parsing honeypot [normal]")
            if ":" in service:
                service, port = service.split(":")  # noqa: PLW2901
                auto = False
                self.options.port = int(port)
                self.server_args["port"] = self.options.port
            elif self.options.port:
                auto = False
            elif self.options.test:
                logger.error(
                    f"server {service} was configured with random port, unable to test..."
                )
                continue
            else:
                auto = True
            self._start_server(service, auto)

    def _start_server(self, service: str, auto: bool | None = None):
        if auto is None:
            auto = self.auto
        server_class = all_servers.get(service.lower())
        if not server_class:
            logger.warning(f"Skipping unknown service {service}")
            return
        server = server_class(**self.server_args)
        if not self.options.test:
            status = server.run_server(process=True, auto=auto)
        else:
            server_timeout(server, service)
            server.kill_server()
            status = False
        self.honeypots.append((server, service, status))

    def _stop_servers(self):
        logger.info("[x] Stopping servers...")
        for server, name, _ in self.honeypots:
            try:
                logger.info(f"[x] Killing {server.__class__.__name__} server")
                server.kill_server()
            except Exception as error:
                logger.exception(f"Error when killing server {name}: {error}")
        logger.info("[x] Please wait few seconds")
        sleep(5)

    def _start_chameleon_mode(self):  # noqa: C901,PLR0912
        logger.info("[x] Chameleon mode")
        if "db_sqlite" in self.config_data["logs"] or "db_postgres" in self.config_data["logs"]:
            logs = self._setup_logging()
        else:
            logger.error("logging must be configured with db_sqlite or db_postgres")
            sys.exit(1)

        if self.options.config != "":
            logger.warning(
                "[x] Config.json file overrides --ip, --port, --username and --password"
            )

        if self.options.port:
            self.options.port = int(self.options.port)
        self.server_args["port"] = self.options.port

        honeypots = self.config_data["honeypots"]
        if isinstance(honeypots, dict):
            logger.info("[x] Parsing honeypot [hard]")
            for honeypot in honeypots:
                self._start_server(honeypot)
        elif isinstance(honeypots, str):
            logger.info("[x] Parsing honeypot [easy]")
            if ":" in honeypots:
                logger.error(
                    "[!] You cannot bind ports with [:] in this mode, "
                    "use the honeypots dict instead"
                )
                sys.exit(1)
            for server in honeypots.split(","):
                self._start_server(server)
        else:
            logger.error(f"[!] Unable to parse honeypots from config: {honeypots}")
            sys.exit(1)

        if self.options.sniffer:
            self._set_up_sniffer()

        if not self.options.test:
            logger.info("[x] Everything looks good!")
            self._stats_loop(logs)
        else:
            self._stop_servers()

    def _setup_logging(self) -> logging.Logger:
        uuid = f"honeypotslogger_main_{str(uuid4())[:8]}"
        if "db_options" in self.config_data:
            drop = "drop" in self.config_data["db_options"]
            logger.info(f"[x] Setup Logger {uuid} with a db, drop is {drop}")
        else:
            drop = True
        return setup_logger("main", uuid, self.options.config, drop)

    def _set_up_sniffer(self):
        sniffer_filter = self.config_data.get("sniffer_filter")
        sniffer_interface = self.config_data.get("sniffer_interface")
        if not sniffer_interface:
            logger.error('If sniffer is enabled, "sniffer_interface" must be set in the config')
            sys.exit(1)
        if not self.options.test and self.options.sniffer:
            _check_interfaces(sniffer_interface)
            if self.options.iptables:
                _fix_ip_tables()
                logger.info("[x] Wait for iptables update...")
                sleep(2)
        self._start_sniffer(sniffer_filter, sniffer_interface)

    def _start_sniffer(self, sniffer_filter, sniffer_interface):
        logger.info("[x] Starting sniffer")
        sniffer = QSniffer(
            filter_=sniffer_filter,
            interface=sniffer_interface,
            config=self.options.config,
        )
        sniffer.run_sniffer(process=True)
        sleep(0.1)
        self.honeypots.append((sniffer, "sniffer", sniffer.server_is_alive()))

    def _stats_loop(self, logs):
        while True:
            try:
                network_stats = {
                    "type": "network",
                    "bytes_sent": net_io_counters().bytes_sent,
                    "bytes_recv": net_io_counters().bytes_recv,
                    "packets_sent": net_io_counters().packets_sent,
                    "packets_recv": net_io_counters().packets_recv,
                }
                logs.info(["system", network_stats])
                load_stats = {
                    server.__class__.__name__: {
                        "memory": Process(server.process.pid).memory_percent(),
                        "cpu": Process(server.process.pid).cpu_percent(),
                    }
                    for server, *_ in self.honeypots
                }
                logs.info(["system", load_stats])
            except Exception as error:
                logger.exception(f"An error occurred during stats logging: {error}")
            sleep(20)


def _fix_ip_tables():
    try:
        logger.info("[x] Fixing iptables")
        Popen(
            "iptables -A OUTPUT -p tcp -m tcp --tcp-flags RST RST -j DROP",
            shell=True,
        )
    except Exception as error:
        logger.exception(f"Could not fix iptables: {error}")


def _check_interfaces(sniffer_interface):
    current_interfaces = "unknown"
    try:
        current_interfaces = " ".join(interfaces())
        if sniffer_interface not in current_interfaces:
            logger.error(
                f"[!] Sniffer interface {sniffer_interface} not found in current interfaces"
            )
            sys.exit(1)
        ip_address = ifaddresses(sniffer_interface)[AF_INET][0]["addr"]
        logger.info(f"[x] Your IP: {ip_address}")
        mac_address = ifaddresses(sniffer_interface)[AF_LINK][0]["addr"]
        logger.info(f"[x] Your MAC: {mac_address}")
    except Exception as error:
        logger.exception(
            f"[!] Unable to detect IP or MAC for [{sniffer_interface}] interface, "
            f"current interfaces are [{current_interfaces}]: {error}"
        )
        sys.exit(1)


def _exit_handler():
    logger.info("[x] Cleaning")
    clean_all()
    sleep(1)


class _ArgumentParser(ArgumentParser):
    def error(self, message):
        logger.error(message)
        self.exit(2, f"Error: {message}\n")


def _parse_args() -> tuple[Namespace, dict[str, str | int]]:
    arg_parser = _ArgumentParser(
        description=(
            "Qeeqbox/honeypots customizable honeypots for monitoring network traffic, bots "
            "activities, and username\\password credentials"
        ),
        usage=SUPPRESS,
    )
    arg_parser._action_groups.pop()
    arg_parser_setup = arg_parser.add_argument_group("Arguments")
    arg_parser_setup.add_argument(
        "--setup",
        help="target honeypot E.g. ssh or you can have multiple E.g ssh,http,https",
        metavar="",
        default="",
    )
    arg_parser_setup.add_argument(
        "--list", action="store_true", help="list all available honeypots"
    )
    arg_parser_setup.add_argument("--kill", action="store_true", help="kill all honeypots")
    arg_parser_setup.add_argument("--verbose", action="store_true", help="Print error msgs")
    arg_parser_optional = arg_parser.add_argument_group("Honeypots options")
    arg_parser_optional.add_argument("--ip", help="Override the IP", metavar="", default="")
    arg_parser_optional.add_argument(
        "--port",
        help="Override the Port (Do not use on multiple!)",
        metavar="",
        default="",
    )
    arg_parser_optional.add_argument(
        "--username", help="Override the username", metavar="", default=""
    )
    arg_parser_optional.add_argument(
        "--password", help="Override the password", metavar="", default=""
    )
    arg_parser_optional.add_argument(
        "--config",
        help="Use a config file for honeypots settings",
        metavar="",
        default="",
    )
    arg_parser_optional.add_argument(
        "--options", type=str, help="Extra options", metavar="", default=""
    )
    arg_parser_optional_2 = arg_parser.add_argument_group("General options")
    arg_parser_optional_2.add_argument(
        "--termination-strategy",
        help="Determines the strategy to terminate by",
        default="input",
        choices=["input", "signal"],
    )
    arg_parser_optional_2.add_argument("--test", action="store_true", help="Test a honeypot")
    arg_parser_optional_2.add_argument(
        "--auto", help="Setup the honeypot with random port", action="store_true"
    )
    arg_parser_chameleon = arg_parser.add_argument_group("Chameleon")
    arg_parser_chameleon.add_argument(
        "--chameleon", action="store_true", help="reserved for chameleon project"
    )
    arg_parser_chameleon.add_argument(
        "--sniffer",
        action="store_true",
        help="sniffer - reserved for chameleon project",
    )
    arg_parser_chameleon.add_argument(
        "--iptables",
        action="store_true",
        help="iptables - reserved for chameleon project",
    )
    argv = arg_parser.parse_args()
    server_args = {
        action.dest: getattr(argv, action.dest, "")
        for action in arg_parser_optional._group_actions
    }
    return argv, server_args


def main_logic():
    argv, server_args = _parse_args()
    manager = HoneypotsManager(argv, server_args)
    manager.main()


if __name__ == "__main__":
    main_logic()



from __future__ import annotations

import struct
from binascii import unhexlify
from contextlib import suppress
from struct import unpack

from twisted.internet import reactor
from twisted.internet.protocol import Protocol, Factory

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)

# authentication with clear text password
AUTH_SIMPLE_BIND = 0x80


class QLDAPServer(BaseServer):
    NAME = "ldap_server"
    DEFAULT_PORT = 389

    def server_main(self):  # noqa: C901
        _q_s = self

        class CustomLDAProtocol(Protocol):
            _state = None

            def connectionMade(self):  # noqa: N802
                self._state = 1
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            @staticmethod
            def parse_ldap_packet(data: bytes) -> tuple[str, str]:
                #                 V
                # 30[20] 0201[02] 60[1b] 0201[03] 04[0a] 7379736261636b757031
                # [80][0a] 7379736261636b757032

                username = ""
                password = ""
                with suppress(IndexError, struct.error):
                    version_offset = data.find(b"\x02\x01\x03")
                    if version_offset > 0:
                        username_start = version_offset + 5
                        username_len = data[username_start - 1]
                        username_end = username_start + username_len
                        username = check_bytes(data[username_start:username_end])
                        auth_type = data[username_end]
                        if auth_type == AUTH_SIMPLE_BIND:
                            if data[username_end + 1] == 0x82:  # noqa: PLR2004
                                # the length of the password needs more than one byte
                                password_start = username_end + 4
                                password_len = unpack(
                                    ">H", data[password_start - 2 : password_start]
                                )[0]
                            else:
                                password_start = username_end + 2
                                password_len = data[password_start - 1]
                            password_end = password_start + password_len
                            password = check_bytes(data[password_start:password_end])

                return username, password

            def dataReceived(self, data):  # noqa: N802
                if self._state == 1:
                    self._state = 2
                    self._check_login(data)
                    self.transport.write(unhexlify(b"300c02010165070a013204000400"))
                elif self._state == 2:  # noqa: PLR2004
                    self._state = 3
                    self._check_login(data)
                    self.transport.write(unhexlify(b"300c02010265070a013204000400"))
                else:
                    self.transport.loseConnection()

            def _check_login(self, data):
                username, password = self.parse_ldap_packet(data)
                if username != "" or password != "":
                    peer = self.transport.getPeer()
                    _q_s.check_login(username, password, ip=peer.host, port=peer.port)

            def connectionLost(self, reason):  # noqa: N802,ARG002
                self._state = None

        factory = Factory()
        factory.protocol = CustomLDAProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from ldap3 import Server, Connection, ALL

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            c = Connection(
                Server(_ip, port=_port, get_info=ALL),
                authentication="SIMPLE",
                user=_username,
                password=_password,
                check_names=True,
                lazy=False,
                client_strategy="SYNC",
                raise_exceptions=True,
            )
            c.open()
            c.bind()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QLDAPServer = QLDAPServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QLDAPServer.run_server()



from __future__ import annotations

from abc import ABC, abstractmethod
from contextlib import suppress
from multiprocessing import Process
from os import getenv
from socket import AF_INET, SOCK_STREAM, socket
from typing import Any
from uuid import uuid4

from psutil import process_iter, TimeoutExpired

from honeypots.helper import (
    get_free_port,
    service_has_started,
    set_local_vars,
    set_up_error_logging,
    setup_logger,
)


class BaseServer(ABC):
    NAME = "base"
    DEFAULT_PORT = 0
    DEFAULT_USERNAME = "test"
    DEFAULT_PASSWORD = "test"

    def __init__(self, **kwargs):
        self.auto_disabled = False
        self.process = None
        self.uuid = f"honeypotslogger_{self.__class__.__name__}_{str(uuid4())[:8]}"
        self.config = kwargs.get("config", "")
        if self.config:
            self.logs = setup_logger(self.__class__.__name__, self.uuid, self.config)
            set_local_vars(self, self.config)
        else:
            self.logs = setup_logger(self.__class__.__name__, self.uuid, None)
        self.ip = kwargs.get("ip", None) or (hasattr(self, "ip") and self.ip) or "0.0.0.0"
        self.port = (
            (kwargs.get("port", None) and int(kwargs.get("port", None)))
            or (hasattr(self, "port") and self.port)
            or self.DEFAULT_PORT
        )
        self.username = (
            kwargs.get("username")
            or (hasattr(self, "username") and self.username)
            or self.DEFAULT_USERNAME
        )
        self.password = (
            kwargs.get("password")
            or (hasattr(self, "password") and self.password)
            or self.DEFAULT_PASSWORD
        )
        self.options = (
            kwargs.get("options", "")
            or (hasattr(self, "options") and self.options)
            or getenv("HONEYPOTS_OPTIONS", "")
            or ""
        )
        self.logger = set_up_error_logging()
        self._server_process: Process | None = None

    def close_port(self):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.settimeout(2)
        if sock.connect_ex((self.ip, self.port)) == 0:
            for process in process_iter():
                with suppress(Exception):
                    for conn in process.connections(kind="inet"):
                        if self.port == conn.laddr.port:
                            process.terminate()
                            try:
                                process.wait(timeout=5)
                            except TimeoutExpired:
                                process.kill()
        with suppress(OSError):
            sock.bind((self.ip, self.port))
            if sock.connect_ex((self.ip, self.port)) != 0:
                return True
        self.logger.error(f"[{self.NAME}]: Could not close port {self.port}")
        return False

    def kill_server(self):
        if self._server_process:
            try:
                self._server_process.terminate()
                self._server_process.join(timeout=5)
            except TimeoutError:
                self._server_process.kill()

    def server_is_alive(self) -> bool:
        return self._server_process and self._server_process.is_alive()

    @abstractmethod
    def server_main(self):
        pass

    def run_server(self, process: bool = False, auto: bool = False) -> bool | None:
        run = False
        if not process:
            self.server_main()
            return None

        if auto and not self.auto_disabled:
            port = get_free_port()
            if port > 0:
                self.port = port
                run = True
        elif self.close_port():
            run = True

        status = self._start_server() if run else "error"

        self.log(
            {
                "action": "process",
                "status": status,
                "src_ip": self.ip,
                "src_port": self.port,
            }
        )

        if status == "success":
            return True
        self.kill_server()
        return False

    def _start_server(self) -> str:
        self._server_process = Process(target=self.server_main)
        self._server_process.start()
        if service_has_started(int(self.port)):
            return "success"
        self.logger.error(f"Server {self.NAME} did not start")
        return "error"

    def check_login(self, username: str, password: str, ip: str, port: int) -> bool:
        status = "success" if self._login_is_correct(username, password) else "failed"
        self.log(
            {
                "action": "login",
                "status": status,
                "src_ip": ip,
                "src_port": port,
                "username": username,
                "password": password,
            }
        )
        return status == "success"

    def _login_is_correct(self, username: str, password: str) -> bool:
        return username == self.username and password == self.password

    def log(self, log_data: dict[str, Any]):
        log_data.update(
            {
                "server": self.NAME,
                "dest_ip": self.ip,
                "dest_port": self.port,
            }
        )
        self.logs.info(log_data)



from __future__ import annotations

import pytest

from honeypots import QOracleServer
from .utils import (
    assert_connect_is_logged,
    connect_to,
    IP,
    load_logs_from_file,
    USERNAME,
    wait_for_server,
)

PORT = "51521"
PROGRAM = "foo"
SERVICE = "bar"


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QOracleServer, "port": PORT}],
    indirect=True,
)
def test_oracle_server(server_logs):
    with wait_for_server(PORT), connect_to(IP, PORT) as connection:
        payload = (
            "\x00\x00\x03\x04\x00\x06\x00\x00\x00\x00\x00\x00\x00\x00\x08\x00E\x00\x01F\xb9\xd9@"
            "\x00@\x06\x81\xd6\x7f\x00\x00\x01\x7f\x00\x00\x01\xbf\xce\x06\x13\xacW\xde\xc0Z\xb5"
            "\x0cI\x80\x18\x02\x00\xff:\x00\x00\x01\x01\x08\n\x1bdZ^\x1bdZ^\x01\x12\x00\x00\x01"
            "\x00\x00\x00\x01>\x01,\x0cA \x00\xff\xff\x7f\x08\x00\x00\x01\x00\x00\xc8\x00J\x00"
            "\x00\x14\x00AA\xa7C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
            "\x00\x00\x00\x00\x00\x00\x00\x00 \x00\x00 \x00\x00\x00\x00\x00\x00\x00\x00\x00\x01"
            f"(DESCRIPTION=(CONNECT_DATA=(SERVICE_NAME={SERVICE})(CID=(PROGRAM={PROGRAM})"
            f"(HOST=xxxxxxxxxxxxxx)(USER={USERNAME}))(CONNECTION_ID=xxxxxxxxxxxxxxxxxxxxxxxx))"
            f"(ADDRESS=(PROTOCOL=tcp)(HOST={IP})(PORT={PORT})))"
        )
        connection.send(payload.encode())
        response, _ = connection.recvfrom(10000)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connect, login = logs
    assert_connect_is_logged(connect, PORT)

    assert login["action"] == "login"
    assert login["data"] == {"local_user": USERNAME, "program": PROGRAM, "service_name": SERVICE}

    assert response == b"\x00\x08\x00\x00\x04\x00\x00\x00"



from __future__ import annotations

from multiprocessing import Process
from pathlib import Path
from time import sleep

import pytest
from vncdotool import api

from honeypots import QVNCServer
from .utils import (
    assert_connect_is_logged,
    IP,
    load_logs_from_file,
    USERNAME,
    PASSWORD,
    wait_for_server,
)

PW_FILE = Path(__file__).parent / "data" / "pw_file"
PORT = "55900"


def _connect_to_vnc(port: str | None = None, password: str | None = None):
    client = api.connect(f"{IP}::{port or PORT}", username=USERNAME, password=password or PASSWORD)
    client.disconnect()


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QVNCServer, "port": PORT}],
    indirect=True,
)
def test_vnc_server(server_logs):
    # This VNC API creates a blocking daemon thread that can't be trivially stopped,
    # so we just run it in a process and terminate that instead
    with wait_for_server(PORT):
        sleep(0.2)  # somehow the server isn't ready sometimes even though the port is occupied
        process = Process(target=_connect_to_vnc)
        process.start()
    process.terminate()
    process.join(timeout=5)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connection, login = logs
    assert_connect_is_logged(connection, PORT)
    assert login["action"] == "login"
    assert login["status"] == "success"
    assert login["username"] == ""
    assert login["password"] == PASSWORD


PORT2 = str(int(PORT) + 1)
SERVER_CONFIG = {
    "honeypots": {
        "vnc": {
            "file_name": str(PW_FILE),
        },
    }
}


@pytest.mark.parametrize(
    "server_logs",
    [{"server": QVNCServer, "port": PORT2, "custom_config": SERVER_CONFIG}],
    indirect=True,
)
def test_wrong_pw(server_logs):
    with wait_for_server(PORT2):
        sleep(0.2)  # somehow the server isn't ready sometimes even though the port is occupied
        process = Process(target=_connect_to_vnc, args=(PORT2, "foo"))
        process.start()
    process.terminate()
    process.join(timeout=5)

    logs = load_logs_from_file(server_logs)

    assert len(logs) == 2
    connection, login = logs
    assert_connect_is_logged(connection, PORT2)
    assert login["action"] == "login"
    assert login["status"] == "failed"
    assert login["username"] == ""
    assert login["password"] == "foo"



import struct
from socket import inet_aton

from twisted.internet import reactor
from twisted.internet.protocol import DatagramProtocol

from honeypots.base_server import BaseServer
from honeypots.helper import check_bytes, server_arguments


class QDHCPServer(BaseServer):
    NAME = "dhcp_server"
    DEFAULT_PORT = 67

    def server_main(self):
        _q_s = self

        class CustomDatagramProtocolProtocol(DatagramProtocol):
            def payload(self, value, message):
                (
                    op,
                    htype,
                    hlen,
                    hops,
                    xid,
                    secs,
                    flags,
                    ciaddr,
                    yiaddr,
                    siaddr,
                    giaddr,
                    chaddr,
                ) = struct.unpack("1s1s1s1s4s2s2s4s4s4s4s16s", message[:44])
                # op, htype, hlen, hops, xid, secs, flags, ciaddr
                response = b"\x02\x01\x06\x00" + xid + b"\x00\x00\x00\x00\x00\x00\x00\x00"
                # yiaddr, siaddr, giaddr, chaddr
                response += (
                    inet_aton(_q_s.dhcp_ip_lease)
                    + inet_aton(_q_s.dhcp_ip)
                    + inet_aton("0.0.0.0")
                    + chaddr
                )
                # sname, file, magic
                response += b"\x00" * 64 + b"\x00" * 128 + b"\x63\x82\x53\x63"
                # options
                response += bytes([53, 1, value])
                response += bytes([54, 4]) + inet_aton(_q_s.dhcp_ip)
                response += bytes([1, 4]) + inet_aton(_q_s.subnet_mask)
                response += bytes([3, 4]) + inet_aton(_q_s.router)
                response += bytes([6, 4]) + inet_aton(_q_s.dns_server)
                response += bytes([51, 4]) + b"\x00\x00\xa8\xc0"  # lease
                response += b"\xff"
                return response

            def parse_options(self, raw):
                options = {}
                tag_name = None
                tag_size = None
                tag = ""
                for b in raw:
                    if tag_name is None:
                        tag_name = b
                    elif tag_name is not None and tag_size is None:
                        tag_size = b
                        tag = ""
                    elif tag_size:
                        tag_size -= 1
                        tag += chr(b)
                        if tag_size == 0:
                            options.update({check_bytes(tag_name): check_bytes(tag)})
                            tag_name = None
                            tag_size = None
                            tag = ""
                return options

            def datagramReceived(self, data, addr):  # noqa: N802
                try:
                    mac_address = struct.unpack("!28x6s", data[:34])[0].hex(":")
                except struct.error:
                    mac_address = "None"
                data = self.parse_options(data[240:])
                data.update({"mac_address": mac_address})
                _q_s.log(
                    {
                        "action": "query",
                        "status": "success",
                        "src_ip": addr[0],
                        "src_port": addr[1],
                        "data": data,
                    }
                )

        reactor.listenUDP(
            port=self.port, protocol=CustomDatagramProtocolProtocol(), interface=self.ip
        )
        reactor.run()

    def test_server(self, ip=None, port=None):
        pass


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        qdhcpserver = QDHCPServer(
            ip=parsed.ip, port=parsed.port, options=parsed.options, config=parsed.config
        )
        qdhcpserver.run_server()



from contextlib import suppress

from twisted.internet import reactor
from twisted.internet.protocol import Factory
from twisted.words import service

from honeypots.base_server import BaseServer
from honeypots.helper import (
    server_arguments,
    check_bytes,
)


class QIRCServer(BaseServer):
    NAME = "irc_server"
    DEFAULT_PORT = 6667

    def server_main(self):
        _q_s = self

        class CustomIRCProtocol(service.IRCUser):
            def connectionMade(self):  # noqa: N802
                _q_s.log(
                    {
                        "action": "connection",
                        "src_ip": self.transport.getPeer().host,
                        "src_port": self.transport.getPeer().port,
                    }
                )

            def handleCommand(self, command, prefix, params):  # noqa: N802
                if "capture_commands" in _q_s.options:
                    _q_s.log(
                        {
                            "action": "command",
                            "data": {
                                "command": check_bytes(command),
                                "prefix": check_bytes(prefix),
                                "params": check_bytes(params),
                            },
                            "src_ip": self.transport.getPeer().host,
                            "src_port": self.transport.getPeer().port,
                        }
                    )
                service.IRCUser.handleCommand(self, command, prefix, params)

            def dataReceived(self, data: bytes):  # noqa: N802
                try:
                    service.IRCUser.dataReceived(self, data)
                except UnicodeDecodeError:
                    _q_s.logger.debug(
                        f"[{_q_s.NAME}]: Could not decode data as utf-8: {data.hex(' ')}"
                    )

            def irc_unknown(self, prefix, command, params):
                pass

            def irc_NICK(self, prefix, params):  # noqa: N802,ARG002
                username = check_bytes("".join(params))
                password = check_bytes(self.password)
                peer = self.transport.getPeer()
                _q_s.check_login(username, password, ip=peer.host, port=peer.port)

        factory = Factory()
        factory.protocol = CustomIRCProtocol
        reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
        reactor.run()

    def test_server(self, ip=None, port=None, username=None, password=None):
        with suppress(Exception):
            from warnings import filterwarnings

            filterwarnings(action="ignore", module=".*socket.*")
            from socket import socket, AF_INET, SOCK_STREAM

            _ip = ip or self.ip
            _port = port or self.port
            _username = username or self.username
            _password = password or self.password
            c = socket(AF_INET, SOCK_STREAM)
            c.connect((_ip, _port))
            c.setblocking(False)
            c.send(f"PASS {_password}\n".encode())
            c.close()


if __name__ == "__main__":
    parsed = server_arguments()
    if parsed.docker or parsed.aws or parsed.custom:
        QIRCServer = QIRCServer(
            ip=parsed.ip,
            port=parsed.port,
            username=parsed.username,
            password=parsed.password,
            options=parsed.options,
            config=parsed.config,
        )
        QIRCServer.run_server()
